CAPI: Connection-minimal Abduction using Prime Implicates 
==========================================================

# Description

The repository contains the implementation of the method for 
performing connection-minimal abduction for EL TBoxes, as described in 
the paper "Connection-minimal Abduction in EL via translation FOL"
published at SOQE 2021.

* The folder "spass" contains the source for spass
* The folder "java-component" contains Java source for
  * Transformation of abduction problems to SPASS compatible input
  * Collection of prime implicates in the SPASS output
  * Construction of the hypotheses for the original abduction problem from the generated prime implicates
   
	
# Dependencies

 - mvn, available from https://maven.apache.org/
 - Java Version >=9
 - C compiler

# Installation Instructions

* Compile the Java component:

        cd java_component
        mvn package
	cp target/capi-java-component-0.2-SNAPSHOT-jar-with-dependencies.jar ..
	cd ..

* Compile the SPASS component:

  	cd spass
        make clean
        make
	cp SPASS ..
	cd ..	

# Usage

CAPI is used as follows from the command line to generate hypotheses for a missing
entailed concept inclusion:

	./generate-hypotheses.sh OWL-FILE SUBCLASS_IRI SUPER_CLASS_IRI [TIME_LIMIT=10] [POST-PROCESSING-OPTIONS]


where OWL-FILE points to an ontology to be used as background knowledge
and SUBCLASS_IRI and SUPERCLASS_IRI determine the observation (the missing
entailment to be fixed or explained), and the optional argument TIME_LIMIT sets
the time limit to the prime implicate generation \(default is 10 seconds\).

The list of hypotheses is then printed to the standard output.


It is furthermore possible, to perform some optimizations to the result by specifying different options in POST-PROCESSING-OPTIONS.
In this case, the TIME_LIMIT has to be given. The available post-processing options are:

* simplify-conjunctions 
  - replace conjunctions by shorter ones that are equivalent w.r.t. the ontology

* remove-redundant-axioms
  - remove from each hypothesis axioms that are entailed by the ontology and the remaining axioms, thus shorter hypotheses
    that are still equivalent w.r.t. the original ontology

* semantically-ordered
  - sort the hypotheses so that the semantically weaker ones are listed first. 



# Example:

	./generate-hypotheses.sh pizza.owl http://www.co-ode.org/ontologies/pizza/pizza.owl#Margherita http://www.co-ode.org/ontologies/pizza/pizza.owl#MeatyPizza 10 simplify-conjunctions semantically-ordered

	Outputs axioms that may be added to the ontology to make Margherita become a MeatyPizza
  


# SPASS details
The key SPASS flags are as follows

* `-SOS` applies the set-of-support strategy that is guaranteed to be complete and enforces R1 
* `-BoundVars=1` constrains inferences such that clauses with more than one variable are not generated, as needed to enforce R2.
* `-BoundStart=n -BoundLoops=1 -BoundMode=2` can be used to limit the depth of Skolem terms to `n` the bound from R3, that can be computed during the preprocessing.
* `-TimeLimit=p` enforces a soft time limit of `p` seconds after which we are guaranteed to recover all prime implicates already found in a generated file by using additionally the flags `-PKept` and `-FPModel`.

The others flags passed to SPASS are essentially technical flags used to ensure that
- only standard inferences are applied following a breadth-first strategy regarding the depth of terms in the implicates generated (`-Sorts=0 -Auto=0 -RFSub -RBSub -ISRe -ISFc -WDRation=1`).
- we silence the output generation (`-PBDC`)
- we also turn off strong Skolemization (`-CNFStrSkolem=0 -CNFOptSkolem=0`) that is not compatible with our translation to generate all ground prime implicates.


# Authors


[Fajar Haifani](mailto:fhaifani@mpi-inf.mpg.de)

[Patrick Koopmann](mailto:patrick.koopmann@tu-dresden.de) (primary contact for owl2spass)

[Sophie Tourret](mailto:stourret@mpi-inf.mpg.de)

[Christoph Weidenbach](weidenbach@mpi-inf.mpg.de) (primary contact for SPASS)


# Version History

This version of SPASS is based on SPASS 3.9 with the following changes:
- the generation of clauses is done iteratively per skolem term depth
- the stack size was increased for
  - the number of established bindings
  - sequential retrieval operations
- the test for timelimit is done outside of the main loop

# License

This software is distributed under the BSD-3-Clause license. See the LICENSE file for more details.

