package de.tu_dresden.lat.capi.cmd;

import de.tu_dresden.lat.capi.experiments.AbductionProblem;
import de.tu_dresden.lat.capi.ontologyTools.ELFilter;
import de.tu_dresden.lat.capi.owl2spass.OWL2SpassConverter;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.*;
import org.semanticweb.owlapi.model.parameters.Imports;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;

public class TranslateAbductionProblem {
    public static void main(String[] args) throws IOException {
        if(args.length!=4)
            printUsageInfoAndExit();

        File file = new File(args[0]);
        if(!file.exists()){
            System.out.println("Cannot file ontology file "+file);
            printUsageInfoAndExit();
        }

        PrintWriter printWriter = null;

        try {
            OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
            OWLOntology ontology = manager.loadOntologyFromOntologyDocument(file);

            ELFilter.deleteNonELAxioms(ontology);

            OWLDataFactory factory = manager.getOWLDataFactory();

            OWLClass subClass = factory.getOWLClass(IRI.create(args[1]));
            OWLClass superClass = factory.getOWLClass(IRI.create(args[2]));

            if(!ontology.getSignature(Imports.INCLUDED).contains(subClass)) {
                System.out.println("sub class " + subClass+" does not occur in "+file+" - right IRI?");
                printUsageInfoAndExit();
            }
            if(!ontology.getSignature(Imports.INCLUDED).contains(subClass)) {
                System.out.println("super class "+superClass+" does not occur in "+file+" - right IRI?");
                printUsageInfoAndExit();
            }

            OWLSubClassOfAxiom observation = factory.getOWLSubClassOfAxiom(subClass, superClass);
            AbductionProblem abductionProblem = new AbductionProblem(ontology, observation);

            System.out.println("Using ontology: "+file);
            System.out.println(" and observation "+observation);

            OWL2SpassConverter converter = new OWL2SpassConverter(true);

            printWriter = new PrintWriter(new File(args[3]));

            converter.convertAbductionProblem(abductionProblem, printWriter, args[0],"skolem.bound");

            converter.saveNameMap(new File(args[0]+".nameMap"));

        } catch (OWLOntologyCreationException e) {
            e.printStackTrace();
            printUsageInfoAndExit();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            printUsageInfoAndExit();
        } catch (OWL2SpassConverter.TranslationException e) {
            e.printStackTrace();
            printUsageInfoAndExit();
        }

        if(printWriter!=null)
            printWriter.close();

    }

    private static void printUsageInfoAndExit(){
        System.out.println("Translates abduction problems to SPASS input files");
        System.out.println("Usage: ");
        System.out.println(TranslateAbductionProblem.class.getCanonicalName() + "OWL-FILE SUBCLASS_IRI SUPERCLASS_IRI OUTPUT_FILE");
        System.out.println();
        System.out.println("Example: ");
        System.out.println(TranslateAbductionProblem.class.getCanonicalName() + " pizza.owl " +
                "http://www.co-ode.org/ontologies/pizza/pizza.owl#Margherita " +
                "http://www.co-ode.org/ontologies/pizza/pizza.owl#MeatyPizza " +
                "problem.spass");
        System.out.println();
        System.out.println("A map from SPASS predicate names to OWL class IRIs is saved under OWL-FILE.nameMap.");
        System.out.println();
        System.exit(1);
    }
}
