package de.tu_dresden.lat.capi.implicateMatching;

import org.semanticweb.owlapi.model.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;

public class OWLConverter {

    private final OWLDataFactory factory;
    private final Map<String, OWLClass> nameMap;

    public OWLConverter(OWLDataFactory factory, File nameMapFile) throws IOException {
        this.factory=factory;
        nameMap = parseNameMap(factory, nameMapFile);
    }

    public OWLClass asOWLClass(ClassName name) {
        if(name.toString().equals("A_TOP"))
            return factory.getOWLThing();
        return nameMap.get(name.toString());
    }

    public OWLSubClassOfAxiom asOWLAxiom(Subsumption subsumption) {
        return factory.getOWLSubClassOfAxiom(
                factory.getOWLObjectIntersectionOf(
                        subsumption.getLHS()
                                .stream()
                                .map(this::asOWLClass)),
                factory.getOWLObjectIntersectionOf(
                        subsumption.getRHS()
                                .stream()
                                .map(this::asOWLClass)
                )
        );
    }

    private static Map<String, OWLClass> parseNameMap(OWLDataFactory factory, File nameMapFile) throws IOException {
        Map<String, OWLClass> map = new HashMap<>();
        BufferedReader in = new BufferedReader(new FileReader(nameMapFile));

        in.lines().forEach(line -> {
            System.out.println("Reading "+line);
            line = line.trim();
            int pos = line.indexOf(" -> ");
            map.put(line.substring(0, pos),
                    factory.getOWLClass(
                            IRI.create(
                                    line.substring(pos+5,line.length()-1))));
        });
        in.close();
        return map;
    }
}
